/* splice: splice stereo input files together with configurable fades
 *
 * Parameters:
 *    p0 = output start time
 *    p1 = input start time
 *    p2 = input duration
 *    p3 = amplitude multiplier
 *    p4 = fade-in ramp duration
 *    p5 = fade-out ramp duration
 *    p6 = apply DC blocking filter [optional] (0: No, 1: Yes) <default: No>
 *
 *    Assumes function table 1 holds the overall amplitude envelope.
 *    Use makegen(1, .....) or setline() for this. If the envelope is
 *    missing, splice will pretend it holds all 1's.
 *
 *    Assumes function table 2 holds the fade-in shape.
 *    Assumes function table 3 holds the fade-out shape.
 *    These shapes should ramp in the appropriate direction -- splice
 *    doesn't check.
 *
 *    Because of the way cmix constructs function arrays, you should insure
 *    that the last slot of the fade-out shape contains a zero. For example,
 *
 *        makegen(3, 5, 1000, 1, 999, 0, 1, 0)
 *
 *    Fade-in dur plus fade-out dur must not exceed input duration.
 *
 *    The overall amplitude envelope is calculated at the global RTcmix
 *    control rate, which you can change with reset(). This has no effect
 *    on the fade shapes, which are calculated on every sample.
 *
 *    Input and output files must be stereo.
 *
 *    John Gibson (jgg9c@virginia.edu), 2/26/99
 */
load("splice")

system("rm -f splice1.snd")
system("F2 splice1.snd")
output("splice1.snd")

input("astereo.snd")

/* Fade-in ramp shape, from 0 to 1 */
makegen(2, 7, 1000, 0, 1000, 1)

/* Fade-out ramp shape, from 1 to 0, and insuring that last val is 0 */
makegen(3, 7, 1000, 1, 999, 0, 1, 0)

outskip = 0
inskip = 0
dur = dur(0)
amp = 1.0
fadeindur = .01
fadeoutdur = .01
dcblock = 0

splice(outskip, inskip, dur, amp, fadeindur, fadeoutdur, dcblock)

