#include <ugens.h>
#include <sfheader.h>
#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>

/* aminst -- amplitude modulation synthesis
*
*  p0 = start
*  p1 = duration
*  p2 = amplitude
*  p3 = carrier frequency (hz)
*  p4 = modulator frequency (hz)
*  p5 = stereo spread <0-1> [optional]
*  assumes function table 1 is the amplitude envelope
*  assumes function table 2 is the modulation envelope
*  function table 3 is the carrier waveform
*  function table 4 is the modulator waveform
*
*/

extern SFHEADER      sfdesc[NFILES];

double
aminst(p,n_args)
float p[];
int n_args;
{
	long i,nsamps,n,m,count,smps = 0;
	float amp,aamp,maamp;
	float phasecar,sicar,phasemod,simod;
	float tval1,tval2,out[2];
	float *cartable;
	int lencar;
	float *modtable;
	int lenmod;
	float *amparr,amptabs[2];
	int lenamp;
	float *mamparr,mamptabs[2];
	int lenmamp;
	int j,skip,outchans;

	nsamps = setnote(p[0],p[1],1);

	amparr = floc(1);
	if (amparr == NULL)
		die("aminst", "You haven't made the amp envelope.");
	lenamp = fsize(1);
	tableset(p[1], lenamp, amptabs);

	mamparr = floc(2);
	if (mamparr == NULL)
		die("aminst", "You haven't made the modulation envelope.");
	lenmamp = fsize(2);
	tableset(p[1], lenmamp, mamptabs);

	cartable = floc(3);
	if (cartable == NULL)
		die("aminst", "You haven't made the carrier waveform.");
	lencar = fsize(3);
	sicar = p[3] * (float)lencar/SR;

	modtable = floc(4);
	if (modtable == NULL)
		die("aminst", "You haven't made the modulator waveform.");
	lenmod = fsize(4);
	simod = p[4] * (float)lenmod/SR;

	outchans = sfchans(&sfdesc[1]);
	skip = SR/(float)resetval;
	j = 0;
	amp = p[2];
	phasecar = phasemod = 0.0;
	for (i = 0; i < nsamps; i++) {
		while (!j--) {
			aamp = tablei(smps,amparr,amptabs) * amp;
			maamp = tablei(smps,mamparr,mamptabs);
			j = skip;
			smps += skip;
			}

		tval1 = oscili(1.0, sicar, cartable, lencar, &phasecar);
		tval2 = tval1 * oscili(1.0, simod, modtable, lenmod, &phasemod);
		out[0] = (tval1*(1.0-maamp)) + (tval2*maamp);
		out[0] *= aamp;

		if (outchans > 1) {
			out[1] = (1.0 - p[5]) * out[0];
			out[0] *= p[5];
			}

		ADDOUT(out,1);
		}
	endnote(1);
}


profile()
{
	UG_INTRO("aminst",aminst);
}

