#include <stdio.h>
#include <ugens.h>
#include <sfheader.h>
#include <macros.h>

#ifdef COMPATIBLE_FUNC_LOCS         /* set in makefile.conf */
  #define AMP_GEN_SLOT     2
  #define WAVET_GEN_SLOT   1
  #define INDEX_GEN_SLOT   3
#else
  #define AMP_GEN_SLOT     1        /* so that we can use setline instead */
  #define WAVET_GEN_SLOT   2
  #define INDEX_GEN_SLOT   3
#endif

extern SFHEADER      sfdesc[NFILES];

/* fminst -- simple fm instrument
*
*  p0 = start time
*  p1 = duration
*  p2 = pitch of carrier (hz or oct.pc)
*  p3 = pitch of modulator (hz or oct.pc)
*  p4 = fm index low point
*  p5 = fm index high point
*  p6 = amp
*  p7 = stereo spread (0-1) <optional>
*  function slot 1 is the amp envelope, slot 2 is oscillator waveform, 
*     slot 3 is index guide
*/

double fminst(p,n_args)
float *p;
{

	int nsamps, i;
	float sicar, simod, val, out[2];
	float *indexenv, *sine, *ampenv;
	int lensine, lenind;
	float indtabs[2], amptabs[2],  modphs, carphs;
	float index, amp, diff;
	int skip, j;
	int chans;
	extern int resetval;

	/* writes on cmix file 1 */
	nsamps = setnote(p[0],p[1],1);

	sine = floc(WAVET_GEN_SLOT);
	if (sine == NULL)
		die("fminst", "You need to store a waveform in function %d.",
					WAVET_GEN_SLOT);
	lensine = fsize(WAVET_GEN_SLOT);

	if (p[2] < 15.0) { /* oct.pc trap */
		sicar = cpspch(p[2]) * lensine/SR; /* convert cps to si */
		}
	else {
		sicar = p[2] * lensine/SR;
		}

	if (p[3] < 15.0) {
		simod = cpspch(p[3]) * lensine/SR;
		}
	else {
		simod = p[3] * lensine/SR;
		}

	ampenv = floc(AMP_GEN_SLOT);
	if (ampenv) {
		int lenamp = fsize(AMP_GEN_SLOT);
		tableset(p[1], lenamp, amptabs);
	}
	else
		advise("fminst", "Setting phrase curve to all 1's.");

	indexenv = floc(INDEX_GEN_SLOT);
	if (indexenv == NULL)
		die("fminst", "You haven't made the index guide function (table %d).",
				INDEX_GEN_SLOT);
	lenind = fsize(INDEX_GEN_SLOT);
	tableset(p[1], lenind, indtabs);

	p[4] *= simod;
	diff = (p[5] * simod) - p[4];

	chans = sfchans(&sfdesc[1]);

	modphs = carphs = 0;
	skip = SR/(float)resetval;

	for(i=j=0; i<nsamps; i++) {
		if (!j--) {
			if (ampenv)
				amp = table(i, ampenv, amptabs) * p[6];
			else
				amp = p[6];
			index = diff * tablei(i, indexenv, indtabs) + p[4];
			j = skip;
		}

		val = oscil(index,simod,sine,lensine,&modphs); 
		/* osciln allows negative sampling increment */
		out[0] = osciln(amp,sicar+val,sine,lensine,&carphs);

		if (chans > 1) { /* split stereo files between the channels */
			out[1] = (1.0 - p[7]) * out[0];
			out[0] *= p[7];
		}

		ADDOUT(out,1);   /* additive writes */
	}
	endnote(1);
}


profile()
{
	UG_INTRO("fminst",fminst);
}

