#include "size.h"

#define MAXTERMS 33
#define MACH1 1080.0
#define IWRAP(x,y) (((x)>=(y))?(x)-(y):(((x)<0)?(x)+(y):(x)))
#define FABS(x) (((x)<0.0)?-(x):(x))
#define ATAN(x) ATANARRAY[(int)((x)*RADPT2)+512]
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/* #include <malloc.h> */
#include <ugens.h>
#include <sfheader.h>
#include "nobug.h"

extern SFHEADER sfdesc[NFILES];

/* This is the main routine that is called by cmix. Its arguments are as
   follows:

   move(outskip, inskip, dur, amp, recdist, rvbamp, inchan)

   The first 3 are as in mix.  "amp" is the amplitude factor for the input
   sig. If "recdist" is positive, location of source is entered in polar
   coordinates: rho (feet), theta (degrees from front, clockwise). If "rec-
   dist" is neg., loc. entered in cartesian: x, y (in feet, listener at 0,0).
   "recdist" is distance in feet between the "mikes" or "ears". "Rvbamp" is fac.
   for reverberation, between 0 and 1.
*/
extern float rvb_time;          /* From space.c, for time limit. */
extern int tapsize;             /* For allocating space for tap delay */
extern float rholoc[SIZE], thetaloc[SIZE];
extern int cartflag;            /* From path command--coordinate type */
extern int space_called;        /* Flag to be sure setup was done */
extern double mindiff;
extern double SINARRAY[1025], COSARRAY[1025], ATANARRAY[1025];
int earflag;
int UseMikes = 0;

/* defined in movelib.c */
extern void rvb_reset(double *tapdel);
extern int roomtrig(double A, double B, double H, double rho[2][13],
                                                        double theta[2][13]);
extern double binaural(double R, double T, double X, double Y, double H,
                                                double *rho, double *theta);
extern void airfil_set(double rho[2][13], int flag, double airdata[2][13][3]);
extern void earfil_set(double Theta[2][13], int flag,
                       double Fircoeffs[2][13][MAXTERMS],
                       double Firtaps[2][13][MAXTERMS+1]);
extern void mike_set(double Thetas[2][13], double MikeAmps[2][13]);
extern void get_tap(int intap, double outlocs[2][13], double Sig[2][13],
                                                            double *tapdel);
extern void ear(double Sig[2][13], long N, double Fircoeffs[2][13][MAXTERMS],
                                           double Firtaps[2][13][MAXTERMS+1]);
extern void air(double Sig[2][13], double airdata[2][13][3]);
extern void wall(double Sig[2][13]);
extern void mike(double Sig[2][13], double MikeAmps[2][13]);

/* defined in RVB.c */
extern void RVB(double *input, double *output, long counter);



double move(float p[], int n_args)
{
   double Airdata[2][13][3];
   double Rho[2][13], Theta[2][13], Sig[2][13], MikeAmps[2][13];
   double Firtaps[2][13][MAXTERMS + 1], Fircoeffs[2][13][MAXTERMS];
   double R_old, T_old, dist;
   double SIN(), COS(), wrap();
   double outlocs[2][13];
   long tap_set();
   register long nsamps, tapcount = 0, N, i, Nend;
   int flag, intap, z, j, inchan;
   double rvbsig[2], roomsig[2];
   float input[4], outbox[4], rvbamp, dur, tabr[2], tabt[2], amp;
   float *array, tabs[2];       /* for amp envelope */
   double *tapdel;

   if (n_args != 6)
      die("move", "Wrong number of args.");
   if (!space_called)
      die("move", "You must call setup routine 'space' first.");

   dur = p[2];
   dur = ((dur < 0.0) ? ((1.0 - dur) - p[1]) : dur);
   setnote(p[1], dur, 0);           /* set input file */
   nsamps = setnote(p[0], dur, 1);  /* set output file */
   amp = p[3];
   dist = p[4];

   inchan = (int) p[6];
   if (inchan >= sfchans(&sfdesc[0]))
      die("move", "You asked for channel %d of a %d-channel file.",
                                                inchan, sfchans(&sfdesc[0]));
   if (sfchans(&sfdesc[1]) != 2)
      die("move", "Output file must be stereo.");

   /* allocate space for tap delay based on room dimensions */

   tapdel = (double *) calloc((unsigned) (tapsize + 8), sizeof(double));
   if (tapdel == NULL)
      die("move", "Not enough memory for tap delay!");

   /* flag for use of ear filters */
   earflag = (!UseMikes && dist < 0.8 && dist != 0.0);
   if (earflag)
      advise("move", "Running in binaural mode.");

   z = SR / (float) resetval;   /* the control rate for amp curve */
   j = 0;

   array = floc(1);
   if (array) {
      int amplen = fsize(1);
      tableset(dur, amplen, tabs);  /* set up table for amp */
   }
   else
      advise("move", "Setting phrase curve to all 1's.");

   rvbamp = p[5];               /* amp factor for reverberated sig */
   flag = 1;                    /* reset all filters, etc. */
   tableset(dur, SIZE, tabr);
   tableset(dur, SIZE, tabt);
   rvb_reset(tapdel);           /* resets reverb & tap delay */
   intap = 0;                   /* the starting element in tap delay */
   R_old = -100000.0;
   T_old = 0.0;

   /* the processing loop for the input signal */
   advise("move", "Processing input signal...");

   for (N = 0; N < nsamps; N++) {
      double R, T, diff1, diff2;

      R = tablei(N, rholoc, tabr);
      T = tablei(N, thetaloc, tabt);
      diff1 = cartflag ? (R_old - R) : (R_old * SIN(T_old) - R * SIN(T));
      diff2 = cartflag ? (T_old - T) : (R_old * COS(T_old) - R * COS(T));

      if (FABS(diff1) > mindiff || FABS(diff2) > mindiff) {
         R_old = R;
         T_old = T;
         if (roomtrig(R, T, dist, Rho, Theta))
            return -1.0;

         /* set taps, return max samp */
         tapcount = tap_set(Rho, outlocs, earflag);
         airfil_set(Rho, flag, Airdata);
         if (earflag)
            earfil_set(Theta, flag, Fircoeffs, Firtaps);
         else
            mike_set(Theta, MikeAmps);
      }

      if (!GETIN(input, 0))
         break;                 /* get input samp until EOF */

      while (!j--) {
         if (array)
            amp = tablei(N, array, tabs) * p[3];
         else
            amp = p[3];
         j = z;
      }

      if (intap >= tapsize)
         intap -= tapsize;                       /* input tap location */

      tapdel[intap] = input[inchan] * amp;       /* add to delay */
      get_tap(intap, outlocs, Sig, tapdel);      /* get delayed samps */
      if (earflag)
         ear(Sig, N, Fircoeffs, Firtaps);        /* binaural angle filters */
      air(Sig, Airdata);                         /* air absorpt. filters */
      wall(Sig);                                 /* wall absorpt. filters */
      roomsig[0] = roomsig[1] = 0.0;
      for (i = 1; i < 13; ++i) {                 /* sum reflected signals */
         roomsig[0] += Sig[0][i];                /* as input for RVB      */
         roomsig[1] += Sig[1][i];
      }
      /* scale reverb input by amp factor */
      roomsig[0] *= rvbamp;
      roomsig[1] *= rvbamp;
      RVB(roomsig, rvbsig, N);                   /* run through reverberator */

      if (!earflag) {
         roomsig[0] = roomsig[1] = 0.0;          /* zero these again */
         mike(Sig, MikeAmps);                    /* do cardioid mike effect */
         for (i = 1; i < 13; ++i) {              /* sum reflected signals */
            roomsig[0] += Sig[0][i];
            roomsig[1] += Sig[1][i];
         }
      }

      /* sum the direct, early refl. & reverbed sigs  */
      outbox[0] = Sig[0][0] + roomsig[0] + rvbsig[0];
      outbox[1] = Sig[1][0] + roomsig[1] + rvbsig[1];

      ADDOUT(outbox, 1);
      flag = 0;                 /* turn off reset flag */
      intap++;
   }

   /* the processing loop for the tap delay signal */
   advise("move", "Flushing delay lines...");
   nsamps = (N < nsamps) ? N : nsamps;           /* if EOF reached above */
   Nend = nsamps + tapcount;

   for (N = nsamps; N < Nend; N++) {
      if (intap >= tapsize)
         intap -= tapsize;
      tapdel[intap] = 0.0;
      get_tap(intap, outlocs, Sig, tapdel);
      if (earflag)
         ear(Sig, N, Fircoeffs, Firtaps);
      air(Sig, Airdata);
      wall(Sig);
      roomsig[0] = roomsig[1] = 0.0;
      for (i = 1; i < 13; ++i) {
         roomsig[0] += Sig[0][i];
         roomsig[1] += Sig[1][i];
      }
      roomsig[0] *= rvbamp;
      roomsig[1] *= rvbamp;
      RVB(roomsig, rvbsig, N);
      if (!earflag) {
         roomsig[0] = roomsig[1] = 0.0;
         mike(Sig, MikeAmps);
         for (i = 1; i < 13; ++i) {
            roomsig[0] += Sig[0][i];
            roomsig[1] += Sig[1][i];
         }
      }
      outbox[0] = Sig[0][0] + roomsig[0] + rvbsig[0];
      outbox[1] = Sig[1][0] + roomsig[1] + rvbsig[1];
      ADDOUT(outbox, 1);
      intap++;
   }

   /* clean out the reverb unit */
   advise("move", "Flushing reverb...");
   i = 0;
   N = Nend;
   Nend = N + (rvb_time + 0.25) * SR;

   while (N < Nend) {
      roomsig[0] = roomsig[1] = 0.0;
      RVB(roomsig, rvbsig, N);
      outbox[0] = rvbsig[0];
      outbox[1] = rvbsig[1];
      ADDOUT(outbox, 1);
      i++;
      N++;
   }
   printf("\n");
   endnote(1);
   cfree((char *) tapdel);

   return 0.0;
}

