#include <ugens.h>
#include <sfheader.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h> 
#include <math.h>
#define IWRAP(x,y) (((x)>=(y))?(x)-(y):(((x)<0)?(x)+(y):(x)))

extern float    SR;
extern SFHEADER sfdesc[NFILES];
extern struct stat sfst[NFILES];

static float *array, tabs[2];      /* for amp envelope */


double
trans (p,n_args)
    float *p;
    int n_args;
{
    double increment, counter = 0.0, frac, amp;
    double voldsig[4], oldsig[4], newsig[4], dur, interval, interp();
    float input[4], sig[4];
    register int getflag = 1, incount = 1, i, j, z;
    register long int insamps, outsamps, outcount = 0;
    register int nchans = sfchans(&sfdesc[0]);

/*  p0 = outskp  p1 = inskp  p2 = dur (-tmnd)  p3 = intrvl of trnsp. p4 = amp */

/*    SR = sfsrate(&sfdesc[0]); */
    if(n_args != 5)
    {
         fprintf(stderr, "Wrong number of args for trans!\n");
         return;
    }
    interval = octpch(p[3]);  /* convert interval to lin octave */
    increment = cpsoct(10.0+interval)/cpsoct(10.0);  /* the samp incr. */
    if (p[2] == 0.0)
        p[2] = ((float)(sfst[0].st_size - getheadersize(&sfdesc[0]))
                 /(float)sfclass(&sfdesc[0])/(float)sfchans(&sfdesc[0])
                 /sfsrate(&sfdesc[0])) - p[1];

    dur = (p[2]<0.0) ? ((-1.0*p[2])-p[1]) : p[2];     /* timeend if < 0 */
    insamps = setnote(p[1], dur, 0);                 /* set input file */
    outsamps = setnote(p[0], (dur/increment), 1);    /* set output file */

    z = SR/(float)resetval;             /* the control rate for amp curve */
    j = 0;

    array = floc(1);
    if (array) {
        int len = fsize(1);
        tableset(dur/increment, len, tabs);       /* set up table for amp */
    }
    else {
        advise("trans", "Setting phrase curve to all 1's.");
        amp = p[4];
    }

    GETIN(input,0);
    for(i=0; i < nchans; i++) oldsig[i] = input[i];
    GETIN(input,0);
    for(i=0; i < nchans; i++) newsig[i] = input[i];

    /* the processing loop for the input signal */

    while (outcount++ < outsamps)
    {
        while (getflag)
        {
            GETIN(input,0);                        /* get input samp */ 
            for(i=0; i < nchans; i++)
            {
                voldsig[i] = oldsig[i];
                oldsig[i] = newsig[i];
                newsig[i] = input[i];
            }
            incount++;  
            if ((counter - (float)incount) < 0.5) getflag = 0;
        }
        frac = counter - incount + 2.0;   /* The interp value */
        while (!j--)
        {
            if (array)
               amp = tablei(outcount,array,tabs) * p[4];
            j = z;
        }
        for (i=0; i < nchans; i++)
            sig[i] = interp(voldsig[i], oldsig[i], newsig[i], frac) * amp;
        ADDOUT(sig, 1);
        counter += increment;               /* keeps track of interp pointer */
        if ((counter - (float)incount) >= -0.5)
            getflag = 1;                    /* Ready for samp */
    }
    endnote(1);
}


double interp(y0,y1,y2,t)
    register double t, y0, y1, y2;
{
    register double hy2, hy0, a, b, c;
    
    a = y0;
    hy0 = y0/2.0;
    hy2 =  y2/2.0;
    b = (-3.0 * hy0) + (2.0 * y1) - hy2;
    c = hy0 - y1 + hy2;

    return(a + b*t + c*t*t);
}


profile()
{
    UG_INTRO("trans", trans);
}

