#include "../../H/ugens.h"
#include "../../H/sfheader.h"
#include <stdio.h>

#ifdef COMPATIBLE_FUNC_LOCS
  #define AMP_GEN_SLOT     2
  #define WAVET_GEN_SLOT   1
#else
  #define AMP_GEN_SLOT     1        /* so that we can use setline instead */
  #define WAVET_GEN_SLOT   2
#endif

extern SFHEADER sfdesc[NFILES];

/* wavetable: a simple instrument
*
*  p0=start_time
*  p1=duration
*  p2=amplitude
*  p3=frequency or oct.pc
*  p4=stereo spread (0-1) <optional>
*  function slot 1 is amp envelope, slot 2 is waveform
*/

double wavetable(p,n_args)
float p[];
int n_args;
{
	int j,k,nsamps,len,alen;
	float out[2];
	float si, *wavetable, *amptable, phase;
	float amp;
	int cdown,chans,skip;
	float tabs[2];


	nsamps = setnote(p[0],p[1],1);

	wavetable = floc(WAVET_GEN_SLOT);
	if (wavetable == NULL)
		die("wavetable", "You need to store a waveform in function %d.",
							WAVET_GEN_SLOT);
	len = fsize(WAVET_GEN_SLOT);

	amptable = floc(AMP_GEN_SLOT);
	if (amptable) {
		alen = fsize(AMP_GEN_SLOT);
		tableset(p[1], alen, tabs);
	}
	else
		advise("wavetable", "Setting phrase curve to all 1's.");

	if (p[3] < 15.0) /* oct.pc trap */
		p[3] = cpspch(p[3]);
	/* si= freq * size /SR */
	si = p[3] * (float)len/SR;    /* sampling increment */

	chans = sfchans(&sfdesc[1]); /* get file number of channels */
	skip = (int)(SR/(float)resetval);
	cdown = 0;
	phase = 0;
	for (k = 0; k < nsamps; k++ ) {
		while (!cdown--) {
			if (amptable)
				amp = table(k, amptable, tabs) * p[2];
			else
				amp = p[2];
			cdown = skip;
			}
		out[0] = oscili(amp, si, wavetable, len, &phase);

		if (chans > 1) { /* split stereo files between the channels */
			out[1] = (1.0 - p[4]) * out[0];
			out[0] *= p[4];
			}

		ADDOUT(out, 1);
		}
	endnote(1);

	return 0.0;
}


int
profile()
{
	UG_INTRO("wavetable",wavetable);
	return 0;
}

