#include <iostream.h>
#include "heap.h"

rtQueue::rtQueue()
{
  head=NULL;
  tail=NULL;
  size=0;
}

// Return the number of elements on the rtQueue

int rtQueue::getSize()
{
  return size;
}

// Return the starting sample chunk of the top Instrument

int rtQueue::nextChunk()
{
  return head->Inst->chunkstart;
}

// Push an element to end of the rtQueue

void rtQueue::push(Instrument *newInst)
{
  rtQElt *newElt;  // create new rtQElt
  rtQElt *tempElt; // BGG: for queue insertion
  newElt = new rtQElt;
  newElt->Inst = newInst;
  newElt->next = NULL;
  newElt->prev = NULL;

  if (head == NULL)  // if first item on rtQueue
    head = tail = newElt;
  else if(tail->Inst->chunkstart <= newElt->Inst->chunkstart) {
    // append to the end of the rtQueue
    tail->next = newElt;
    newElt->prev = tail;
    tail = newElt;
  }
  else { // BGG: we have to insert this one
	tempElt = tail;
	while(tempElt->Inst->chunkstart > newElt->Inst->chunkstart) {
		if (!tempElt->prev) { // BGG: we're at the head
			break;
			}
		tempElt = tempElt->prev;
		}

	if(tempElt->prev) {  // BGG: "normal" insertion
		newElt->next = tempElt->next;
		newElt->prev = tempElt;
		(tempElt->next)->prev = newElt;
		tempElt->next = newElt;
		}
	else { // BGG: put at the head
		newElt->next = tempElt;
		newElt->prev = NULL;
// BGG:  I thought I would have to do the following line here,
//	but experience shows that it dumps core and does bad things
//		(tempElt->next)->prev = newElt;
		head = newElt;
		}
  }
  size++;
}

// Pop an element of the top of the rtQueue

Instrument *rtQueue::pop() 
{
  rtQElt *tQelt;
  Instrument *retInst;
  tQelt = head;
  if (!head) {
    cerr << "ERROR: attempt to pop empty rtQueue\n";
    return NULL;
  }
  retInst = head->Inst;
  head = head->next;
  delete tQelt;
  size--;
  return retInst;

}





