#ifdef RTUPDATE

//#include <globals.h>
#include "../H/ugens.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <rtupdate.h>

extern double gen1(struct gen *gen, char *sfname);
extern double gen2(struct gen *gen);
extern double gen3(struct gen *gen);
extern double gen5(struct gen *gen);
extern double gen6(struct gen *gen);
extern double gen7(struct gen *gen);
extern double gen9(struct gen *gen);
extern double gen10(struct gen *gen);
extern double gen17(struct gen *gen);
extern double gen18(struct gen *gen);
extern double gen20(struct gen *gen);
extern double gen24(struct gen *gen);
extern double gen25(struct gen *gen);

static int npgens = 1;           /* total number of gens so far */
/* Note: This was 0, but then non-existant gen funcs were not caught.  -JGG */

static int npipaths = 0;


/* ok, TOTGENS is the absolute total number of gens we can have in
   a given run, MAXGENS is the number of simultaneous gens we can
   have.  TOTGENS should probably be replaced by an appropriate malloc */

#define TOTGENS 10000
#define MAXGENS 300

// used to store pgen information
float *parrays[TOTGENS];
int sizeof_parray[TOTGENS];
int p_goto[MAXGENS];   /* this is used to guarantee unique parrays in rtcmix */

// This function allows the user to stop using a given pf path array and start
// from scratch rather than simply appending the new data to the old
double unset_pfield_path(float p[], int n_args, double pp[])
{
	int inst, param;
	inst = (int)p[0];
	pi_goto[inst] = npipaths++;  // create new instrument array by incrementing
								 // the pi_goto counter

	if(npipaths > MAXNUMINSTS) // question: should the user be warned when this
		npipaths = 1;          // happens?
	return 0.0;
}

// This function is used to associate a gen slot to a given name for a gen.
// This function is called when a user uses a name for a pgen call instead of
// a number.  It sets up the mapping between a gen name and its associated slot
// number.
int set_gen_num(char *name)
{ 
	struct inst_list *tmp;
	int genslot;

	// first gen called so need to allocate memory for the head of the list
	if(genlist == NULL)
	{	
		genlist = (struct inst_list *) malloc(sizeof(struct inst_list));
		genlist->next = NULL;
		genlist->num = curgen++;
	    genlist->name = name;
	}

	tmp = genlist;

	// check to see if the gen already exists in the list and iterates to the
	// end of the list
	while((strcmp(name, tmp->name) != 0) && (tmp->next != NULL))
	{
		tmp = tmp->next;
	}
	// if the gen already exists, set genslot to the pre-existing slot number
	// for that gen
	if(strcmp(name, tmp->name) == 0)
	{
		genslot = tmp->num;
	}
	// otherwise give this gen name a new gen number and place it at the end
	// of the list
	else
	{	  
		tmp->next = (struct inst_list *) malloc(sizeof(struct inst_list));
		tmp = tmp->next;
		tmp->next = NULL;
		tmp->num = curgen++;
		tmp->name = name;
		genlist->next = tmp;
		genslot = tmp->num;
	}
	return genslot;
}

// This function is used to set up the mapping between instrument names and 
// their associated instrument tag numbers.  This function will set the 
// number for a given instrument, based on the name that it is passed, and 
// will associate either the next availiable number, or the tag number 
// specified by the user 
int set_inst_num(char *name, int tag)
{
	struct inst_list *tmp;
	int instnum;

	// first inst called so need to allocate memory for the head of the list
	if(ilist == NULL)
	{	
		ilist = (struct inst_list *) malloc(sizeof(struct inst_list));
		ilist->next = NULL;

		// this flag exists so that this function can be called either with 
		// an explicit tag number, or with the default "next in line" tag
		if(tag == -1)
			ilist->num = curinst++;
		else
			ilist->num = tag;
	    ilist->name = name;
	}
	tmp = ilist;
	
	// check to see if the inst already exists in the list and iterates to the
	// end of the list
	while((strcmp(name, tmp->name) != 0) && (tmp->next != NULL))
	{
		tmp = tmp->next;
	}

	
	// if the instrument already exists assign the instruments tag number to 
	// the pre-existing number already given to that instrument
	if(strcmp(name, tmp->name) == 0)
	{
		instnum = tmp->num;
	}
	// otherwise give it a new tag number and add it to the end of the list
	// (the -1 flag is used in the same way it is in the initial case
	else
	{	  
		tmp->next = (struct inst_list *) malloc(sizeof(struct inst_list));
		tmp = tmp->next;
		tmp->next = NULL;
		if(tag == -1)
			tmp->num = curinst++;
		else
			tmp->num = tag;
		tmp->name = name;
		instnum = tmp->num;
	}

	return instnum;
}

// this function allows the user to specify the tag number that will be used 
// for a specific instrument.  It is important to note that this will only set
// the tag if that instrument has not already been given a tag.  This prevents
// the user from overwriting pre-existing tags.  (basically, to do anything, 
// this function must be called before the first instance of the instrument)
double set_itag_num(float p[], int n_args, double pp[])
{
	char* name;
	int num, tname;
	
	tname = (int) pp[0];
	name = (char*) tname;
	num = (int) p[1];
	set_inst_num(name, num);
	return 0.0;
}

// this function allows the user to specify time-value pairs that represent 
// a path that they would like a certain parameter of an instrument to take
// the program interpolates between these values using a gen type (p0) that is
// specified by a call to the function pgen (found below).  
// p0 = gen type (specified by a name or a number)
// p1 = instrument (specified by a name or a number)
// p2 = parameter to change
// p3,p4 - pn,pn+1 = time value pairs representing the values that the 
// specified parameter should be at the specified time.  
double inst_pfield_path(float p[], int n_args, double pp[])
{
	int inst, param;
	int i, j;
	char* string_test;
	
	inst = (int)pp[1];
	param = (int)p[2];

	// note:  this is a hack used to determine if the user is passing an 
	// instrument tag number or a string identifier to the function.  If it 
	// gets a number larger than the maximum number of instruments than it
	// assumes that it is a string

	if(inst > MAXNUMINSTS)
	{  
		string_test = (char*)inst;  
	    inst = set_inst_num(string_test, -1);  
	}

	// tests to see if this is the first call to inst_pfield_path
	if(pi_goto[inst] == -1)
		pi_goto[inst] = npipaths++;

	if(npipaths > MAXNUMINSTS)
		npipaths = 1;
	
	igen_type[pi_goto[inst]][param][numinstcalls[pi_goto[inst]][param]] 
                                   = (int)pp[0];

	// this is the same hack as above to test to see if the gen type that is
	// being passed to this function is a string or a number
	if(igen_type[pi_goto[inst]][param][numinstcalls[pi_goto[inst]][param]] 
       > MAXGENS)
	{
		string_test = 
   (char*)igen_type[pi_goto[inst]][param][numinstcalls[pi_goto[inst]][param]];

		igen_type[pi_goto[inst]][param][numinstcalls[pi_goto[inst]][param]]
              = set_gen_num(string_test);

	}
	 
	// since this specifies the number of time value *pairs* the / 2 is 
	// necessary
    piarray_size[pi_goto[inst]][param][numinstcalls[pi_goto[inst]][param]] 
                                   = (n_args - 3) / 2;
	

	// i = 0 for the first call to this function, and is equal to the total
	// number of elements that have been added so far otherwise
	if(numinstcalls[pi_goto[inst]][param] <= 0)
 	  	i = 0;  
  	else   
		i = cum_piarray_size[pi_goto[inst]][param]; 

	cum_piarray_size[pi_goto[inst]][param] += 
          piarray_size[pi_goto[inst]][param][numcalls[pi_goto[inst]][param]];


	j = 0;
	for(; i < cum_piarray_size[pi_goto[inst]][param]; ++i)
	{
		pipath[pi_goto[inst]][param][i][0] = p[(2 * j) + 3];
	 	pipath[pi_goto[inst]][param][i][1] = p[(2 * j) + 4];

		// this verifies that all time values specified by the user are in 
		// ascending chronological order.  If not, the program exits with an
		// error message.  This seemed to me to be the best way to handle this 
		// problem, but feel free to change it if you think some sort of 
		// automatic action that keeps the program moving would be better
		// (I have one commented out under the "die" statement)
		if(i > 0)
		{
			if(pipath[pi_goto[inst]][param][i-1][0] 
                                   >= pipath[pi_goto[inst]][param][i][0])
			{
               die("pipath", "invalid pipath specified...exiting program\n");

//				pfpath[tag][param][i-1][0] = pfpath[tag][param][i][0] - 0.0001;
			}
		}
		
		j++;
    }

	numinstcalls[pi_goto[inst]][param]++;
	return 0.0;
}


// this function allows the user to specify time-value pairs that represent 
// a path that they would like a certain parameter of a specific instance of 
// an instrument (note) to take the program interpolates between these 
// values using a gen type (p0) that is specified by a call to the function 
// pgen (found below).  
// p0 = gen type (specified by a name or a number)
// p1 = note tag (specified by a number)
// p2 = parameter to change
// p3,p4 - pn,pn+1 = time value pairs representing the values that the 
// specified parameter should be at the specified time. 
double note_pfield_path(float p[], int n_args, double pp[])
{
	int tag, param;
	int i, j;
	char* string_test;
	tag = (int)p[1];
	param = (int)p[2];


	


	gen_type[tag][param][numcalls[tag][param]] = (int)pp[0];
				   
	// this is the same hack as above to test to see if the gen type that is
	// being passed to this function is a string or a number
	if(gen_type[tag][param][numcalls[tag][param]] 
       > MAXGENS)
	{
		string_test = 
   (char*)gen_type[tag][param][numcalls[tag][param]];

		gen_type[tag][param][numcalls[tag][param]]
              = set_gen_num(string_test);

	}

	// since this specifies the number of time value *pairs* the / 2 is 
	// necessary
    parray_size[tag][param][numcalls[tag][param]] = (n_args - 3) / 2;
	
	// i = 0 for the first call to this function, and is equal to the total
	// number of elements that have been added so far otherwise
	if(numcalls[tag][param] <= 0)
 	  	i = 0;  
  	else   
		i = cum_parray_size[tag][param]; 


	cum_parray_size[tag][param] += 
                parray_size[tag][param][numcalls[tag][param]];

	j = 0;
	for(; i < cum_parray_size[tag][param]; ++i)
	{
		pfpath[tag][param][i][0] = p[(2 * j) + 3];
	 	pfpath[tag][param][i][1] = p[(2 * j) + 4];


		// this verifies that all time values specified by the user are in 
		// ascending chronological order.  If not, the program exits with an
		// error message.  This seemed to me to be the best way to handle this 
		// problem, but feel free to change it if you think some sort of 
		// automatic action that keeps the program moving would be better
		// (I have one commented out under the "die" statement)
		if(i > 0)
		{
			if(pfpath[tag][param][i-1][0] >= pfpath[tag][param][i][0])
			{
               die("npfpath", "invalid pfpath specified...exiting program\n");

//				pfpath[tag][param][i-1][0] = pfpath[tag][param][i][0] - 0.0001;
			}
		}

		j++;
    }
	numcalls[tag][param]++;

	return 0.0;
}

// This function defines an envelope that can later be used by the 
// note_pfield_path and inst_pfield_path functions to define the interpolation
// between time-value pairs.  It has remarkable similarities to makegen 
// p0 = tag number
// p1 = gen no
// p2 = size
// p3 = args
double pgen(float p[], int n_args, double pp[])
{
	int    gen_tag, genno;
    double retval = -1.0;
    struct gen gen;
	char* string_test;
	gen_tag = (int) pp[0];
    genno = (int) p[1];
	
	// this is the same hack as above to test to see if the gen type that is
	// being passed to this function is a string or a number
	if(gen_tag > MAXGENS)
	{  
		string_test = (char*)gen_tag;  
	    gen_tag = set_gen_num(string_test);  
	}

	if (gen_tag < 0)
		gen_tag = -gen_tag;
    if (gen_tag >= MAXGENS)
		die("makegen", "no more simultaneous gens available!");

	
    /* pgen now creates a new function for *every* call to it - this
       is so that we can guarantee the correct version of a given function
       at run-time during RT operation.  the p_goto[] array keeps track
       of where to map a particular function table number during the
       queueing of RT Instruments
    */
    p_goto[gen_tag] = npgens;

	gen.size = p[2];

	// FIXME: Should we really be using valloc here?  -JGG
    if (genno != 1) {    /* gen1 must allocate its own memory */
		float *block = (float *) valloc((unsigned) gen.size * FLOAT);
        if (block == NULL)
			die("makegen", "Not enough memory for function table %d."
                , gen_tag);
        parrays[p_goto[gen_tag]] = block;
    }
    npgens++;

	sizeof_parray[p_goto[gen_tag]] = p[2];
	gen.nargs = n_args - 3;
    gen.pvals = p + 3;
    gen.array = parrays[p_goto[gen_tag]];
    gen.slot = (int) p[0];   /* get from pfield, to preserve negative "flag" */

	switch (genno) {
		case 25:
			retval = gen25(&gen);
			 break;
		case 24:
			retval = gen24(&gen);
			 break;
		case 20:
			retval = gen20(&gen);
            break;
		case 18:
			retval = gen18(&gen);
			break;
        case 17:
			retval = gen17(&gen);
			break;
        case 10:
			retval = gen10(&gen);
			break;
		case 9:
			retval = gen9(&gen);
			break;
		case 7:
			retval = gen7(&gen);
			break;
		case 6:
			retval = gen6(&gen);
			break;
		case 5:
			retval = gen5(&gen);
			break;
		case 3:
			retval = gen3(&gen);
			break;
		case 2:
			retval = gen2(&gen);
			break;
		case 1:
        {
			char *sfname = (char *) ((int) pp[3]);
            gen.pvals = p + 2;    /* gen1() needs size pfield */
            gen.nargs++;
            retval = gen1(&gen, sfname);
            if (retval != -1.0) {
				/* We don't know these before calling gen1, so get them now. */
                parrays[p_goto[gen_tag]] = gen.array;
                sizeof_parray[p_goto[gen_tag]] = gen.size;
            }
         }
         break;
      default:
         die("makegen", "There is no gen%d.", (int) p[1]);
   }
	return retval;
}

// This function returns the array of gen data created by pgen.  It abstracts
// the p_goto construct from the caller, and verifies that valid data exists
float *ploc(int gen_tag)
{
   int index = p_goto[gen_tag];
   int size = sizeof_parray[index];

   if (size == 0) {
      if (gen_tag == 1)
         return NULL;
      else {
         fprintf(stderr, "You haven't allocated function %d yet!\n", gen_tag);
         closesf();
      }
   }
   return parrays[index];
}

// returns the size of the function specified by gen_tag 
int psize(int gen_tag)
{
	if(!sizeof_parray[p_goto[gen_tag]]) {
	       fprintf(stderr,"psize: You haven't declared function %d yet!\n"
                   ,gen_tag);
		closesf();  
	}
	return(sizeof_parray[p_goto[gen_tag]]);
}

// Used to abstract the pi_goto construct from the caller.  Simply returns an
// index into the pi_path array 
int piloc(int instnum)
{
	int index = pi_goto[instnum];
	return index;
} 

#endif /* RTUPDATE */
