/* RTcmix  - Copyright (C) 2000  The RTcmix Development Team
   See ``AUTHORS'' for a list of contributors. See ``LICENSE'' for
   the license to this software and for a DISCLAIMER OF ALL WARRANTIES.
*/
#include <globals.h>
#include <ugens.h>      /* for die() */
#include <stdio.h>
#include <string.h>

extern void set_rtoutput_clobber(int state);    /* defined in rtoutput.c */

typedef enum {
  AUDIO,
  CLOBBER,
  REPORT_CLIPPING,
  CHECK_PEAKS,
  FULL_DUPLEX,
} ParamType;

typedef struct {
   char       arg[32];   
   ParamType  type;
   int        value;
} Param;

static Param param_list[] = {
   { "AUDIO_ON",   AUDIO, 1},
   { "AUDIO_OFF",  AUDIO, 0},
   { "CLOBBER_ON",  CLOBBER, 1},
   { "CLOBBER_OFF",  CLOBBER, 0},
   { "REPORT_CLIPPING_ON",  REPORT_CLIPPING, 1},
   { "REPORT_CLIPPING_OFF",  REPORT_CLIPPING, 0},
   { "CHECK_PEAKS_ON",  CHECK_PEAKS, 1},
   { "CHECK_PEAKS_OFF",  CHECK_PEAKS, 0},
   { "FULL_DUPLEX_ON",  FULL_DUPLEX, 1},
   { "FULL_DUPLEX_OFF",  FULL_DUPLEX, 0},
};

static int num_params = sizeof(param_list) / sizeof(Param);

double set_option(float *p, short nargs, double pp[]) {
  int i, j,anint, matched; 
  char *arg;
    
  for (i = 0; i < nargs; i++) {
    anint = (int)pp[i];
    arg = (char *)anint;
    
    matched = 0;
    for (j = 0; j < num_params; j++) {
      if (strcasecmp(param_list[j].arg, arg) == 0) {
	matched = 1;
	break;
      }
    }
    if (!matched) {
      fprintf(stderr, "setoption: unrecognized argument \"%s\"\n", arg);
      return -1;
    }
    
    switch (param_list[j].type) {
    case AUDIO:
      play_audio = param_list[j].value;
      break;
    case CLOBBER:
      set_rtoutput_clobber(param_list[j].value);
      break;
    case REPORT_CLIPPING:
      report_clipping = param_list[j].value;
      break;
    case CHECK_PEAKS:
      check_peaks = param_list[j].value;
      break;
    case FULL_DUPLEX:
      full_duplex = param_list[j].value;
      if (full_duplex && rtsetparams_called)
         die("set_option", "Turn on full duplex BEFORE calling rtsetparams.");
      break;
      
    }
  }
  return 0.0;
}


